"""
_summary_

:return: _description_
:rtype: _type_
"""

import urllib.parse
from pathlib import Path

import numpy as np
import pandas as pd
import requests
from bs4 import BeautifulSoup

import open_geodata as geo

# import os
# import pandas as pd
# from open_geodata import geo
# from paths import input_path, output_path_tab


class CETESB:
    def __init__(self) -> None:
        self.df_agencias_municipios = None

    def get_agencia_ambiental(self, municipio: str):
        """
        Obtem informações de um dado município
        https://licenciamento.cetesb.sp.gov.br/municipioss.asp?muni=SANTOS

        :param mun: Nome do Município (de acordo com a grafia da CETESB)
        :type mun: string
        :return: informações sobre a Agência Ambiental que atua no sminucípio selecionado
        :rtype: dictionary
        """
        # Município
        mun_url = municipio.encode('ISO-8859-1')
        mun_url = urllib.parse.quote(mun_url)
        print(f'{municipio} > {mun_url}')

        # Set URL
        url = f'https://licenciamento.cetesb.sp.gov.br/municipioss.asp?muni={mun_url}'

        # Get Data
        r = requests.get(url)
        soup = BeautifulSoup(r.content, 'html.parser')
        # time.sleep(1)

        # Get all relevant data
        table = soup.find('table', attrs={'width': '300'})
        rows = table.find_all('tr')

        # Create List Adress
        list_end = [municipio]
        for row in rows:
            text = row.get_text()
            text = ' '.join(text.split())
            text = text.strip()
            list_end.append(text)

        # Rename List
        for i in ['Bairro:', 'Cidade:', 'CEP:', 'Fone:', 'Fax:', 'E-mail:']:
            list_end = np.char.replace(list_end, i, '')
        list_end = [x.strip() for x in list_end]

        # Format Variables
        cep = '{:08d}'.format(int(list_end[5]))
        tel = list_end[6].strip().replace(' ', '').replace('-', ' ')
        fax = list_end[7].strip().replace(' ', '').replace('-', ' ')

        # Create Dictionary
        return {
            'municipio_cetesb': list_end[0],
            'agencia': list_end[1],
            'endereco': list_end[2].split('nº')[0].strip(),
            'numero': list_end[2].split('nº')[1].strip(),
            'bairro': list_end[3],
            'municipio_sede': list_end[4].title(),
            'cep': '{}-{}'.format(cep[:5], cep[5:8]),
            'telefone': f'({tel[:2]}) {tel[3:7]}-{tel[7:]}',
            'fax': f'({fax[:2]}) {fax[3:7]}-{fax[7:]}',
            'email': list_end[8],
            'url': url,
        }

    def get_all_agencias(self):
        #
        package_path = Path(__file__).parents[3].absolute()
        data_path = package_path / 'data'
        print(data_path)

        # Read Dataframe
        # df = pd.read_csv(data_path / 'tab' / 'sp_cetesb' / 'tab_municipios.csv')
        df = geo.data.load_dataset(db='sp', name='tab.municipio_cetesb')
        df = geo.data.load_dataset(db='sp', name='tab.municipio_cetesb')
        # print(df.head())

        # Create empty list
        list_addresses = []

        # df = df[0:4]  # For tests only
        for index, row in df.iterrows():
            # Create Small Dictionary
            dict_df = {
                'id_municipio': row['id_municipio'],
            }

            # Get Addresses
            list_address = self.get_agencia_ambiental(row['municipio_cetesb'])
            list_address = {**dict_df, **list_address}
            list_addresses.append(list_address)

        # Create Table from dictionarys
        return pd.DataFrame.from_dict(list_addresses)

    def get_all_agencias_adjusted(self):
        # Obtem Tabela Padrão com os nomes de municípios
        df_mun = geo.data.load_dataset(db='sp', name='tab.municipio_nome')

        # Obtem Tabela da CETESB
        df_cetesb = self.get_all_agencias()

        # Merge
        df = pd.merge(
            df_mun,
            df_cetesb,
            how='left',
            left_on='id_municipio',
            right_on='id_municipio',
        )

        # Ajustes
        # Split
        df[['endereco', 'complemento']] = df['endereco'].str.split(
            ' - ', expand=True
        )

        # Reorder
        cols = df.columns.tolist()
        cols = cols[0:5] + [cols[-1]] + cols[5:-1]
        df = df[cols]

        #
        # Adjust Table
        df.loc[:, 'endereco'] = (
            df['endereco'].astype(str).apply(lambda x: self._rename_nome(x))
        )
        df.loc[:, 'bairro'] = (
            df['bairro'].astype(str).apply(lambda x: self._rename_nome(x))
        )
        df.loc[:, 'municipio_sede'] = (
            df['municipio_sede']
            .astype(str)
            .apply(lambda x: self._rename_nome(x))
        )
        df.loc[:, 'fax'] = (
            df['fax'].astype(str).apply(lambda x: self._rename_nome(x))
        )
        df.loc[:, 'complemento'] = (
            df['complemento'].astype(str).apply(lambda x: self._rename_nome(x))
        )

        # Deleta colunas
        df = df.drop(['municipio_cetesb'], axis=1, inplace=False)

        # Atribui
        if self.df_agencias_municipios is None:
            self.df_agencias_municipios = df
        return df

    def _rename_nome(self, x):
        x = x.title()
        x = x.strip()
        dict_rename = {
            # Nome
            'Av ': 'Av. ',
            'Av. ': 'Avenida ',
            ' Joao ': ' João ',
            'Ten.': 'Tenente',
            # Basics
            ' Com ': ' com ',
            ' Sobre ': ' sobre ',
            ' Da ': ' da ',
            ' De ': ' de ',
            ' Do ': ' do ',
            ' Das ': ' das ',
            ' Dos ': ' dos ',
            ' A ': ' a ',
            ' As ': ' as ',
            ' Ao ': ' ao ',
            ' Aos ': ' aos ',
            ' E ': ' e ',
            ' O ': ' o ',
            ' Os ': ' os ',
            '(11) -': '',
            '( ) -': '',
            'Jd. ': 'Jardim ',
            'V. ': 'Vila ',
            'Sta.': 'Santa',
            'Pq. ': 'Parque ',
            'Res.': 'Residencial ',
            ' Sao ': ' São ',
            'J.S. Marcos': 'Jardim São Marcos',
            'Jardim-': 'Jardim -',
            'Xxiii': 'XXIII',
            'Ii': 'II',
            # Empty
            'None': '',
            'none': '',
        }
        for k, v in dict_rename.items():
            x = x.replace(k, v)
        x = x.replace('  ', ' ')
        return x.strip()

    def _get_ag_mun(self):
        # Se já rodou
        if isinstance(self.df_agencias_municipios, pd.DataFrame):
            df_ag_mun = self.df_agencias_municipios

        # Senão, roda
        else:
            df_ag_mun = self.get_all_agencias_adjusted()
        return df_ag_mun

    def get_only_agencias(self) -> pd.DataFrame:
        df_ag_mun = self._get_ag_mun()

        # Get column's name
        cols = list(df_ag_mun.columns[2:-1])

        # Group
        df = pd.DataFrame(
            df_ag_mun.groupby(cols, dropna=False)['id_municipio'].count()
        )
        df.reset_index(inplace=True)

        # Delta Última Coluna
        df = df.drop(df.columns[len(df.columns) - 1], axis=1, inplace=False)
        return df

    def get_geodataframe(self):

        # Padrão
        gdf = geo.load_dataset(db='sp', name='geo.sp_250k_wgs84')
        gdf = gdf.drop(['municipio_nome'], axis=1, inplace=False)
        gdf['id_municipio'] = gdf['id_municipio'].astype(int)
        gdf['geometry'] = gdf['geometry'].simplify(0.0015)

        # Tabela AgÊncias e Municípios
        df_ag_mun = self._get_ag_mun()

        # Join
        gdf = gdf.merge(df_ag_mun, on='id_municipio', how='left')
        return gdf


if __name__ == '__main__':
    from pathlib import Path

    import pandas as pd

    cetesb = CETESB()
    aa = cetesb.get_agencia_ambiental('Piracicaba')
    print(aa)

    # bb = cetesb.get_all_agencias()
    # print(bb.head())

    cc = cetesb.get_all_agencias_adjusted()
    print(cc.head())



import pprint

import geopandas as gpd
import pytesseract
from pathlib import Path
from SICAR import Polygon, Sicar, State


class SICAR:
    """
    Class to handle the Sicar data provider.
    """

    def __init__(
        self,
        tesseract_path: str | Path,
    ):
        """
        Initialize the Sicar data provider with optional parameters.
        """
        tesseract_path = Path(tesseract_path)
        if tesseract_path.is_file():
            self.tesseract_path = Path(tesseract_path)

            # Set the tesseract command path for pytesseract
            pytesseract.pytesseract.tesseract_cmd = (
                self.tesseract_path.as_posix()
            )

        else:
            raise ValueError(f"Invalid tesseract path: {tesseract_path}")

        # Create Sicar instance
        self.car = Sicar()

    @property
    def release_dates(self):
        """
        Method to retrieve data from the Sicar provider.
        """
        # Implementation for retrieving data goes here
        return self.car.get_release_dates()

    @property
    def list_states(self) -> list[State]:
        return [x for x in State]

    @property
    def list_layers(self) -> list[Polygon]:
        return [x for x in Polygon]

    def download_data(
        self, sigla_estado: State, layer: Polygon, output_path, *args, **kwargs
    ):
        """
        Download data for a specific state.
        """
        n_tentativas = kwargs.get('n_tentativas', 3)
        tentativa = 0

        while tentativa < n_tentativas:
            try:
                self.car.download_state(
                    state=sigla_estado,
                    polygon=layer,
                    folder=output_path,
                )
                print(f"Polygon for {sigla_estado} downloaded successfully.")
                break

            except Exception as e:
                print(f"Error downloading polygon for {sigla_estado}: {e}")
                tentativa += 1


if __name__ == "__main__":
    # Example usage
    # sicar = SICAR(tesseract_path="/usr/bin/tesseract")
    sicar = SICAR(
        tesseract_path=r'C:\\Program Files\\Tesseract-OCR\\tesseract.exe'
    )
    release_dates = sicar.release_dates
    pprint.pprint(release_dates)

    #
    print(sicar.list_states)

    # Example of using the Sicar class to get a state
    # state = sicar.car.get_state(State.ACRE)
    # print(f"State: {state.name}, Code: {state.code}")



"""
sssss

"""

import sidrapy
import pandas as pd


def get_estimated_population(id_municipio):
    """
    Retorna a população estimada
    :param id_municipio:
    :return:
    """
    # Get Table
    df = sidrapy.get_table(
        table_code='6579',
        territorial_level='6',
        ibge_territorial_code=id_municipio,
        period='all',
        header='n',
    )

    # Dict
    dict_col = {
        'D1C': 'id_municipio',
        'D1N': 'municipio_nome',
        'V': 'n_habitantes',
        'D2N': 'ano'
    }

    # Rename Columns
    df.rename(
        dict_col,
        axis=1,
        inplace=True
    )

    # Select Columns
    df = df[[v for k, v in dict_col.items()]]

    # Adjust Columns
    df.sort_values(by=['ano'], inplace=True)
    df['id_municipio'] = pd.to_numeric(df['id_municipio'], errors='coerce')
    df['n_habitantes'] = pd.to_numeric(df['n_habitantes'], errors='coerce')
    df['ano'] = pd.to_numeric(df['ano'], errors='coerce')

    # Results
    return df


if __name__ == '__main__':
    cod_ibge = '3526902'  # Limeira
    pop = get_estimated_population(cod_ibge)
    print(pop)



"""
sssss
"""

import json
import logging
import pkgutil
import shutil
import tempfile
import warnings
from pathlib import Path
from urllib.parse import quote, unquote, urlparse, urlunparse
from zipfile import ZipFile

import geopandas as gpd
import pandas as pd
import pooch
import py7zr
import rarfile

# import importlib.resources
# import pprint
# from more_itertools import one, only


class DB:
    def __init__(self, db='general', project='open_geodata') -> None:
        self.project = project
        self.cache = None

        db_path = Path(__file__).parent / 'db'
        list_dbs = list(db_path.rglob('*.json'))
        list_dbs = [x.stem for x in list_dbs]
        if db not in list_dbs:
            list_dbs_str = '\n'.join(list_dbs)
            raise Exception(
                f'O db deve ser um dos listados abaixo\n{list_dbs_str}'
            )

        json_data = pkgutil.get_data(
            package=self.project, resource=f'db/{db}.json'
        )
        if isinstance(json_data, bytes):
            self.json_raw = json.loads(json_data)
        else:
            raise Exception('Erro!')
        self.json = self._flatten()
        # self._check_keys()

    def _flatten(self):
        flat = {}
        for outer_key, inner_dict in self.json_raw.items():
            for inner_key, value in inner_dict.items():
                new_key = f"{outer_key}.{inner_key}"
                flat[new_key] = value
        return flat

    def _check_keys(self):
        for dict_data in self.json.values():
            if all(key in dict_data for key in ['url', 'hash']):
                pass
            else:
                raise Exception('Falta chaves')

    @property
    def list_data(self):
        return list(self.json.keys())

    def get_base_url(self, name):
        if name not in self.list_data:
            raise Exception('Nome Inválido')

        #
        url = self.json[name]['url']
        scheme = urlparse(url=url).scheme
        netloc = urlparse(url=url).netloc
        path = Path(urlparse(url=url).path).parent.as_posix()
        params = urlparse(url=url).params
        query = urlparse(url=url).query
        fragment = urlparse(url=url).fragment

        url = urlunparse((scheme, netloc, path, params, query, fragment))
        url = quote(url, safe=':/')
        return url

    def _get_hash(self, name):
        if name not in self.list_data:
            raise Exception('Nome Inválido')

        return self.json[name]['hash']

    def _get_filename(self, name):
        if name not in self.list_data:
            raise Exception('Nome Inválido')

        #
        url = self.json[name]['url']
        path = Path(urlparse(url=url).path)
        return unquote(path.name)

    def get_registry(self, name):
        return {self._get_filename(name=name): self._get_hash(name=name)}

    def _create_cache_for_data(self, name):
        self.cache = pooch.create(
            path=pooch.os_cache(project=self.project),
            base_url=self.get_base_url(name=name),
            # version='v1.8.2',
            # version_dev='main',
            registry=self.get_registry(name=name),
        )
        return self.cache

    def get_data(self, name):
        self.cache = self._create_cache_for_data(name=name)
        return self.cache.fetch(fname=self._get_filename(name=name))

    def get_filehash_sha256(self, name):
        filename = self.get_data(name=name)
        return pooch.file_hash(filename, alg="sha256")

    def read_7z(self, name, *args, **kwargs) -> gpd.GeoDataFrame:
        layer = kwargs.get('layer', None)
        file_path_7z = self.get_data(name=name)

        with py7zr.SevenZipFile(file_path_7z, 'r') as archive:
            allfiles = archive.getnames()

            # Quero apenas um arquivo por gpkg
            if len(allfiles) == 1:
                with tempfile.TemporaryDirectory() as temp_dir:
                    temp_dir = Path(temp_dir)
                    archive.extract(path=temp_dir, targets=allfiles)

                    #
                    filename = list(temp_dir.glob('*'))[0]
                    ext = filename.suffix.lower()
                    if ext in ['.gpkg']:
                        return gpd.read_file(filename=filename, layer=layer)

                    else:
                        raise Exception(
                            f'Não tem configuração para extesão {ext}'
                        )

            else:
                raise RuntimeError('.zip tem mais de um gpkg')


def load_dataset(db, name, *args, **kwargs) -> pd.DataFrame | gpd.GeoDataFrame:
    """
    Funções para carregar dados geoespaciais

    :param dataset_name:
    :return:
    """
    #
    shapefile = kwargs.get('shapefile')

    db_obj = DB(db=db)

    # Confere se o nome do objeto está na lista
    list_data = db_obj.list_data
    if name not in list_data:
        list_data_str = '\n'.join(list_data)
        raise Exception(f'Deve ser um dado listado abaixo\n{list_data_str}')

    # Faz download
    filepath = db_obj.get_data(name=name)
    # filename = dataset_name.replace('.', '/')
    # filename = Path(filename)
    # package_path = Path(__file__).absolute().parent
    # data_path = package_path / 'data'
    # file_path = data_path / filename

    # Checa se existe
    # list_shp = get_dataset_names()
    # if dataset_name not in list_shp:
    #     raise RuntimeError(f'"{filename}" not exists')

    # # Checa se existe mais de um
    # if list_shp.count(dataset_name) > 1:
    #     raise RuntimeError(
    #         f'Exists "{list_shp.count(dataset_name)}" datasets named "{dataset_name}"'
    #     )

    # Teste ambos tipos de arquivos
    ext = Path(filepath).suffix.lower()

    # Load por tipo de arquivo
    if ext in ['.7z']:
        return db_obj.read_7z(name=name)

    # Se o arquivo é um
    elif ext in ['.csv']:
        return pd.read_csv(filepath_or_buffer=filepath, **kwargs)

    # Se o arquivo é um
    elif ext in ['.xls', '.xlsx']:
        return pd.read_excel(io=filepath, **kwargs)

    # Se o arquivo é um
    elif ext in ['.gpkg']:
        return gpd.read_file(filename=filepath, *args, **kwargs)

    elif ext in ['.zip']:
        with ZipFile(file=filepath) as zip_obj:
            for info in zip_obj.infolist():
                if Path(info.filename).suffix.lower() == '.shp':
                    try:
                        if shapefile is not None:
                            # O Geopandas permite ler shape especifico em zipfile usando o ! como separador
                            # gpd.read_file(filename='shp_cnuc_2025_03.zip!cnuc_2025_03.shp',
                            return gpd.read_file(
                                filename=f'{filepath}!{shapefile}',
                                *args,
                                **kwargs,
                            )
                        else:
                            return gpd.read_file(
                                filename=filepath, *args, **kwargs
                            )

                    except Exception as e:
                        raise e

            try:
                return pd.read_csv(filepath_or_buffer=filepath)

            except Exception as e:
                raise e

    elif ext in ['.rar']:
        # warnings.warn('Desenvolver!')
        #  pass

        # Salva
        # Cria uma pasta temporária
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_dir = Path(temp_dir)

            # Extrai tudo para a pasta temporária
            with rarfile.RarFile(file=filepath) as rf:
                rf.extractall(temp_dir)

                # Se extrair pasta, move conteudo da pasta
                list_files = list(temp_dir.rglob('*'))
                for file in list_files:
                    # print(Path(file))
                    if file.is_file():
                        try:
                            shutil.move(src=file, dst=temp_dir)
                            # print(file)

                        except Exception as e:
                            print(e)

                # Carrega o shapefile no GeoDataFrame
                list_shp = list(temp_dir.rglob('*.shp'))
                if len(list_shp) == 1:
                    return gpd.read_file(
                        filename=list_shp[0],
                        engine='fiona',
                        driver='ESRI Shapefile',
                    )
                else:
                    print('Definir o que fazer')

    else:
        raise Exception(f'Extensão {ext} não configurada.')


# def _ajust_list_files(list_files):
#     """

#     :param list_files:
#     :return:
#     """
#     # Convert to text
#     list_files = [str(x.as_posix()) for x in list_files]

#     # Splita no . e pega primeira parte
#     list_files = [x.split('.', maxsplit=1)[0] for x in list_files]

#     # Convert Paths Linux
#     list_files = [x.replace('/', '.') for x in list_files]
#     list_files = list(set(list_files))
#     list_files.sort()
#     return list_files


# def _read_7z_file(file_path_7z):
#     if file_path_7z.is_file():


# def get_dataset_names():
#     """
#     fdfddfd

#     """
#     package_path = Path(__file__).absolute().parent
#     data_path = package_path / 'data'
#     list_files = data_path.rglob('*.*')
#     list_files = [x for x in list_files]
#     list_files = [x.relative_to(data_path) for x in list_files]
#     return _ajust_list_files(list_files)


# def load_dataset(dataset_name) -> pd.DataFrame | gpd.GeoDataFrame:
#     """
#     Funções para carregar dados geoespaciais

#     :param dataset_name:
#     :return:
#     """

#     filename = dataset_name.replace('.', '/')
#     filename = Path(filename)
#     package_path = Path(__file__).absolute().parent
#     data_path = package_path / 'data'
#     file_path = data_path / filename

#     # Checa se existe
#     list_shp = get_dataset_names()
#     if dataset_name not in list_shp:
#         raise RuntimeError(f'"{filename}" not exists')

#     # Checa se existe mais de um
#     if list_shp.count(dataset_name) > 1:
#         raise RuntimeError(
#             f'Exists "{list_shp.count(dataset_name)}" datasets named "{dataset_name}"'
#         )

#     # Teste ambos tipos de arquivos
#     file_path_7z = file_path.with_suffix('.7z')
#     file_path_csv = file_path.with_suffix('.csv')

#     # Load por tipo de arquivo
#     if file_path_7z.is_file():
#         return _read_7z_file(file_path_7z)

#     # Se o arquivo é um
#     elif file_path_csv.is_file():
#         return pd.read_csv(file_path_csv)

#     else:
#         print('Não encontrado')


# def get_dataset_from_package(package_name):
#     """
#     Pega dados dos pacotes
#     Os dados preciso estar disponibilizados em .7z ou .csv

#     """
#     # sss
#     package_path = importlib.resources.files(package_name)
#     list_files = [
#         x for x in package_path.rglob('*') if x.suffix in ('.7z', '.csv')
#     ]
#     list_files = [x.relative_to(package_path) for x in list_files]
#     list_files = [x.relative_to('data') for x in list_files]
#     for path in ['input', 'output']:
#         try:
#             list_files = [x.relative_to(path) for x in list_files]
#         except:
#             pass
#     return _ajust_list_files(list_files)


# def load_dataset_from_package(package_name, dataset_name):
#     """ """
#     # Importa o Package
#     package_path = importlib.resources.files(package_name)
#     print(package_path)

#     #
#     filename = dataset_name.replace('.', '/')
#     filename = Path(filename)
#     print(f'O nome do arquivo é {filename}')

#     # Lista de Arquivo
#     list_7zips = list(package_path.rglob(f'{filename}*.7z'))
#     list_csv = list(package_path.rglob(f'{filename}*.csv'))

#     # Pega Valor Único
#     file_path_7z = only(list_7zips)
#     file_path_csv = only(list_csv)

#     # Prints
#     # print(f'A lista de 7zip é: {file_path_7z}')
#     # print(f'A lista de csv é: {file_path_csv}')

#     if file_path_7z is not None and file_path_7z.is_file():
#         return _read_7z_file(file_path_7z)

#     elif file_path_csv is not None and file_path_csv.is_file():
#         return pd.read_csv(file_path_csv)

#     else:
#         print('Não encontrado')


# if __name__ == '__main__':
#     from open_geodata import geo
#     from open_geodata.functions import find_neighbors, share_boundary

#     # # List Geodata
#     list_shp = get_dataset_names()
#     pprint.pprint(list_shp)

#     # # Read Geaodata
#     gdf = load_dataset('geo.sp.sp_250k_wgs84')
#     print(gdf.head())

#     # # List Geodata
#     list_shp = get_dataset_from_package('sp_piracicaba')
#     pprint.pprint(list_shp)

#     gdf = load_dataset_from_package('sp_piracicaba', 'geo.divisa_municipal')
#     print(gdf.head())

# Teste "find_neighbors" attribute table
# gdf = find_neighbors(gdf, 'municipio_nome')

# Teste "find_neighbors" spatial
# gdf_interest = gdf.loc[gdf['id_municipio'] == 3548906]
# gdf = share_boundary(gdf, gdf_interest)

# Results
# print(gdf.head())

# df = load_dataset('tab_municipio_ugrhi')
# print(df.head())

# create_colors(
#     os.path.join('outorgas.gpkg'),
#     col_categories = 'uso'
# )
# for i in list_shp:
#     #print(i.parents[2])
#     a = i.relative_to(i.parents[2])
#     print(a)



# open_geodata
from .converts import coords, files
from .folium_plus import adds, png
from .functions import share_boundary
from .data import DB, load_dataset
from .lyr import base
from .providers import br, sp

__all__ = [
    'share_boundary',
    'base',
    'adds',
    'png',
    'coords',
    'files',
    #'ibge',
]



"""
Funções para trabalhar com dados geoespaciais
"""


def find_neighbors(gdf, column_name):
    """
    Encontre os vizinhos e insere coluna "neighbors" no geodataframe com seus nomes

    :param column_name: Título da Coluna que contém o nome do vizinho
    :param gdf: geodataframe de input
    :return: geodataframe com uma coluna "neighbors", com o nome dos vizinhos
    """
    gdf['neighbors'] = ''

    for index, row in gdf.iterrows():
        neighbors = gdf[gdf['geometry'].touches(
            row['geometry'])][column_name].tolist()
        gdf.at[index, 'neighbors'] = ', '.join(neighbors)
    return gdf


def share_boundary(gdf, gdf_interest):
    """
    Adaptação de https://stackoverflow.com/questions/66153429/selecting-polygons-which-shares-a-boundary-with-target-polygon-in-python-prefera
    :param gdf:
    :param gdf_interest:
    :return:
    """
    if len(gdf_interest['geometry']) == 1:
        gdf_interest = gdf_interest['geometry'].values[0]
    else:
        print('"gdf_interest" tem mais de um polígono')

    return gdf[gdf.geometry.apply(lambda x: x.touches(gdf_interest))]


def get_centroid(gdf):
    gdf['apenasparacentroid'] = 35
    gdf_dissolve = gdf.dissolve(by='apenasparacentroid')
    gdf_centroid = gdf_dissolve.representative_point()
    gdf = gdf.drop('apenasparacentroid', axis=1, inplace=True)
    return [float(gdf_centroid.y), float(gdf_centroid.x)]


if __name__ == '__main__':
    pass



"""
Móodulo para acessar as malhas municipais do IBGE

"""

from pathlib import Path

import geopandas as gpd
import requests


class IBGE:
    def __init__(self):
        pass

    def get_malhas_municipais(self, estado: str = 'SP') -> gpd.GeoDataFrame:
        """
        _summary_


        >>> import open_geodata as geo


        :param url: _description_
        :type url: _type_
        :return: _description_
        :rtype: _type_
        """
        # Obtem dados do IBGE
        gdf = gpd.read_file(
            filename=f'http://geoftp.ibge.gov.br/organizacao_do_territorio/malhas_territoriais/malhas_municipais/municipio_2018/UFs/{estado}/{estado.lower()}_municipios.zip'
        )

        # Renomeia Colunas
        gdf = gdf.rename(
            columns={
                'NM_MUNICIP': 'nome_municipio',
                'CD_GEOCMU': 'id_municipio',
            }
        )
        # Converte o tipo de dado da coluna id_municipio para int
        gdf['id_municipio'] = gdf['id_municipio'].astype(int)

        # Reordena colunas
        gdf = gdf[['id_municipio', 'nome_municipio', 'geometry']]

        return gdf

    def download_malhas_municipais(
        self, estado: str = 'SP', output_path: str | Path = './'
    ) -> None:
        """
        Download das malhas municipais do IBGE para o estado especificado.

        :param estado: Sigla do estado (ex: 'SP' para São Paulo)
        :param output_path: Caminho onde os arquivos serão salvos
        """
        url = f'http://geoftp.ibge.gov.br/organizacao_do_territorio/malhas_territoriais/malhas_municipais/municipio_2018/UFs/{estado}/{estado.lower()}_municipios.zip'
        filename = Path(url).name
        # Download
        r = requests.get(url, allow_redirects=True)
        open(Path(output_path) / filename, 'wb').write(r.content)


if __name__ == "__main__":
    ibge = IBGE()
    gdf = ibge.get_malhas_municipais()
    gdf = gdf.loc[gdf['id_municipio'] == 3504206]
    print(gdf.head())
    print(gdf['id_municipio'] == 3504206)



"""
s

"""

import geopandas as gpd
import seaborn as sns


def create_colors(input_geojson, col_categories):
    """
    Funçao que cria dictionary com colors para layers
    :param input_geojson:
    :param col_categories:
    :return:
    """
    gdf = gpd.read_file(input_geojson)

    list_cols = list(set(gdf.columns))
    if col_categories not in list_cols:
        print('"col_categories" must  be in:')
        print(list_cols)

    # Set palette
    palette_polygon = 'Paired'

    # Get list of unique values
    categories = list(set(gdf[col_categories]))
    categories.sort()

    # See the palette chosed
    pal = sns.color_palette(palette_polygon, n_colors=len(categories))

    # Set dictionary
    color_polygon = dict(zip(categories, pal.as_hex()))
    return color_polygon


if __name__ == '__main__':
    pass



"""
_summary_
"""

import os


dddd = 'https://arquivos.receitafederal.gov.br/dados/cnpj/dados_abertos_cnpj/2025-06/'


if __name__ == "__main__":
    from open_geodata.providers.br.receita_federal.rfb import RFB



"""
Agêncio Nacional de Mineração (ANM) - Brazil
This module provides access to the ANM's geospatial data services.
"""

import shutil
import ssl
import urllib.request
from pathlib import Path

import geopandas as gpd
import pandas as pd


class ANM:
    def __init__(self):
        """
        Initialize the ANM data provider.

        :param cache_dir: Directory to cache downloaded data.
        """

        list_tuples = {
            (
                'Áreas de Bloqueio',
                'https://app.anm.gov.br/dadosabertos/SIGMINE/BLOQUEIO.zip',
            ),
            (
                'Áreas de Proteção de Fonte',
                'https://app.anm.gov.br/dadosabertos/SIGMINE/PROTECAO_FONTE.zip',
            ),
            (
                'Áreas de Servidão',
                'https://app.anm.gov.br/dadosabertos/SIGMINE/AREA_SERVIDAO.zip',
            ),
            (
                'Arrendamentos',
                'https://app.anm.gov.br/dadosabertos/SIGMINE/ARRENDAMENTO.zip',
            ),
            (
                'Reservas Garimpeiras',
                'https://app.anm.gov.br/dadosabertos/SIGMINE/RESERVAS_GARIMPEIRAS.zip',
            ),
            (
                'Processos Minerários Ativos - SP',
                'https://app.anm.gov.br/dadosabertos/SIGMINE/PROCESSOS_MINERARIOS/SP.zip',
            ),
        }

        df = pd.DataFrame(list_tuples, columns=['nome', 'url'])
        self.layers = df

    def get_layers(self, layer: str) -> gpd.GeoDataFrame:
        """
        Get the available layers as a GeoDataFrame.

        :return: GeoDataFrame containing the layer names and URLs.
        """
        ssl._create_default_https_context = ssl._create_unverified_context
        url = self.layers.loc[self.layers['nome'] == layer, 'url'].values[0]
        return gpd.read_file(filename=url)

    def download_file(self, layer: str, output_path: str | Path):
        """
        Download a file from the given URL and save it to the cache directory.

        :param url: URL of the file to download.
        :param filename: Name of the file to save in the cache directory.
        :return: Path to the downloaded file.
        """
        # SSL
        ssl._create_default_https_context = ssl._create_unverified_context

        # Iteração
        for i, row in self.layers.iterrows():
            if layer == row['nome']:
                print(f'Downloading {row["nome"]}...')
                url = row['url']
                url_path = Path(url)
                filepath = Path(output_path) / url_path.name

                # Download
                with (
                    urllib.request.urlopen(url) as r,
                    open(filepath, 'wb') as out_file,
                ):
                    shutil.copyfileobj(r, out_file)


if __name__ == '__main__':
    import tempfile

    import geopandas as gpd

    # Instancia
    amn2 = ANM()

    # Get available layers
    # print(amn2.layers)

    # Download a specific layer
    with tempfile.TemporaryDirectory() as temp_dir:
        temp_dir = Path(temp_dir)

        amn2.download_file(
            layer='Processos Minerários Ativos - SP', output_path=temp_dir
        )

        # List files in the temporary directory
        list_files = list(temp_dir.glob('*'))

        gdf = gpd.read_file(filename=list_files[0])
        print(gdf.head)

    # Get a specific layer as a GeoDataFrame
    gdf = amn2.get_layers(layer='Processos Minerários Ativos - SP')
    print(gdf.head())




# https://stackoverflow.com/questions/33997361/how-to-convert-degree-minute-second-to-degree-decimal

# # Degrees, minutes, seconds to Decimal degrees
# def dms2dd(degrees, minutes, seconds, direction):
#     dd = (float(degrees.replace(',', '.')) +
#           float(minutes.replace(',', '.'))/60 +
#           float(seconds.replace(',', '.'))/(60*60)
#          )
#     if direction == 'E' or direction == 'N':
#         dd *= 1

#     elif direction == 'W' or direction == 'O' or direction == 'S':
#         dd *= -1

#     return dd

# # Decimal degrees to Degrees, minutes, seconds
# def dd2dms(deg):
#     d = int(deg)
#     md = abs(deg - d) * 60
#     m = int(md)
#     sd = (md - m) * 60
#     sd = round(sd, 10)
#     return [d, m, sd]

# # Parse Values
# def parse_dms(dms):
#     coord = re.split('[^\d\w\.,]+', dms)
#     lat = dms2dd(coord[0], coord[1], coord[2], coord[3])
#     return (lat)

# # Single Value
# # DMS to DD
# coordinates_dms = '53°19\'03,208\"S'
# coordinates_dd = parse_dms(coordinates_dms)
# print('A coordenada DMS "{}" foi convertida para coordenada DD "{}"'.format(coordinates_dms, coordinates_dd))

# # DD to DMS
# d, m, s = dd2dms(coordinates_dd)
# print('A coordenada DD "{}" foi convertida em "{}", "{}" e "{}"'.format(coordinates_dd, d, m, s))



"""
Sobreescreve método "to_png" do folium
com objetivo de usar um webdriver que não está localizado
"""

import time

import folium
from folium.utilities import temp_html_filepath
from selenium import webdriver
from selenium.webdriver.firefox.options import Options as FirefoxOptions
from selenium.webdriver.firefox.service import Service as FirefoxService


class MyMap(folium.Map):
    """
    _summary_
    """

    def to_png_custom(self, gecko_path, log_path, delay=3):
        """
        Export the HTML to byte representation of a PNG image.

        Uses selenium to render the HTML and record a PNG. You may need to
        adjust the `delay` time keyword argument if maps render without data or tiles.

        Examples
        --------
        >>> m._to_png()
        >>> m._to_png(time=10)  # Wait 10 seconds between render and snapshot.
        """
        if self._png_image is None:
            options = FirefoxOptions()
            options.add_argument('--headless')

            # Adicionado por mim!
            service = FirefoxService(
                executable_path=gecko_path, log_path=log_path
            )
            driver = webdriver.Firefox(options=options, service=service)

            html = self.get_root().render()
            with temp_html_filepath(html) as fname:
                # We need the tempfile to avoid JS security issues.
                driver.get(f'file:///{fname}')
                driver.maximize_window()
                time.sleep(delay)
                png = driver.get_screenshot_as_png()
                driver.quit()
            self._png_image = png
        return self._png_image


if __name__ == '__main__':

    import io

    from paths import docs_path_imgs, scrapy_path_driver, scrapy_path_logs
    from PIL import Image

    # Cria mapa como objeto
    m = MyMap(location=[-23.9619271, -46.3427499], zoom_start=12)

    # Salva
    img_data = m.to_png_custom(
        gecko_path=scrapy_path_driver / 'geckodriver.exe',
        log_path=scrapy_path_logs / 'geckodriver.log',
        delay=7,
    )
    img = Image.open(io.BytesIO(img_data))
    img.save(docs_path_imgs / 'cetesb_enquadramento.png')



"""
Adicionando funções ao folium
nov.22
"""

import os
import pprint
import webbrowser
from pathlib import Path

import branca as bc
import folium
import pandas as pd


def create_map_multitiles(location=[-23.9619271, -46.3427499], zoom_start=10):
    """
    
    :param tile_service:
    :param location:
    :param zoom_start:
    :return:
    """

    # Create Maps
    m = folium.Map(
        location=location,
        zoom_start=zoom_start,
        tiles=None,
    )

    # Read table with all tiles servers
    package_path = Path(__file__).absolute().parents[1]
    data_path = package_path / 'data'
    tiles_path = data_path / 'tab' / 'folium'
    df = pd.read_csv(tiles_path / 'tiles.csv', index_col=0)

    # Filter some tiles
    # df = df[2:4]
    # df = df.loc[(df['name'] == 'ESRI Satelite') | (df['name'] == 'ESRI Street')]
    # df = df[df['name'].str.startswith(('ESRI'))]
    df = df[df['name'].str.startswith(('Google'))]
    # df = df[df['name'] == 'Google Satellite']

    # Loop
    for index, row in df.iterrows():
        # Create reference to attribution
        attr = row['attribution']
        name = row['name']
        link = row['link']

        # Create multiples tiles layers
        folium.TileLayer(
            tiles=link,
            attr=f'<a href="{attr}" target="blank">{name}</a>',
            name=name,
        ).add_to(m)

    # Results
    return m


def modify_header_legend(m):
    """
    sdsdsdsd
    """
    # Header to Add
    head = """
    {% macro header(this, kwargs) %}
    <script src="https://code.jquery.com/ui/1.12.1/jquery-ui.js"></script>
    <script>$( function() {
        $( ".maplegend" ).draggable({
            start: function (event, ui) {
                $(this).css({
                    right: "auto",
                    top: "auto",
                    bottom: "auto"
                });
            }
        });
    });
    </script>
    {% endmacro %}
    """

    # Add Header
    macro = bc.element.MacroElement()
    macro._template = bc.element.Template(head)
    m.get_root().add_child(macro)

    # CSS to Add (on Header)
    css = """
    {% macro header(this, kwargs) %}
    <style type='text/css'>
      .maplegend {
        position: absolute;
        z-index:9999;
        background-color: rgba(255, 255, 255, 1);
        border-radius: 5px;
        border: 2px solid #bbb;
        padding: 10px;
        font-size:14px;
        right: 10px;
        bottom: 20px;
      }
      .maplegend .legend-title {
        text-align: left;
        margin-bottom: 5px;
        font-weight: bold;
        font-size: 90%;
        }
      .maplegend .legend-scale ul {
        margin: 0;
        margin-bottom: 0px;
        padding: 0;
        float: left;
        list-style: none;
        }
      .maplegend .legend-scale ul li {
        font-size: 80%;
        list-style: none;
        margin-left: 0;
        line-height: 18px;
        margin-bottom: 2px;
        }
      .maplegend ul.legend-labels li span {
        display: block;
        float: left;
        height: 16px;
        width: 30px;
        margin-right: 5px;
        margin-left: 0;
        border: 0px solid #ccc;
        }
      .maplegend .legend-source {
        font-size: 80%;
        color: #777;
        clear: both;
        }
      .maplegend a {
        color: #777;
        }
    </style>
    {% endmacro %}
    """

    # Add CSS (on Header)
    macro = bc.element.MacroElement()
    macro._template = bc.element.Template(css)
    m.get_root().add_child(macro)
    return m


def add_categorical_legend(m, title, color_by_label):
    """
    """

    # Modify Header
    m = modify_header_legend(m)

    # ddd
    body = f"""
    <div id='maplegend {title}' class='maplegend'>
        <div class='legend-title'>{title}</div>
        <div class='legend-scale'>
            <ul class='legend-labels'>"""

    # Loop Categories
    for label, color in color_by_label.items():
        body += f"""
                <li><span style='background:{color}'></span>{label}</li>"""
    body += """
            </ul>
        </div>
    </div>
    """
    # Add Body
    body = bc.element.Element(body)
    m.get_root().html.add_child(body)
    return m


if __name__ == '__main__':
    from sp_piracicaba import lyr

    from open_geodata import geo

    # List Geodata
    list_shp = geo.get_dataset_from_package('sp_piracicaba')
    pprint.pprint(list_shp)

    gdf = geo.load_dataset_from_package('sp_piracicaba', 'geo.macrozonas')
    print(gdf.head())

    # Create Map
    m = create_map_multitiles()

    # Add Layers
    m.add_child(sp_piracicaba.lyr.macrozona())
    # sp_piracicaba.

    # Add Layer Control
    folium.LayerControl('topright', collapsed=False).add_to(m)

    # Add Legend
    colors = {
        'MADE': '#0173b2',
        'MANU': '#de8f05',
        'MAPH': '#029e73',
        'MCU': '#d55e00',
        'MRU': '#cc78bc',
        'MUC': '#ca9161',
    }
    # m = modify_header_legend(m)
    m = add_categorical_legend(m, title='Legenda', color_by_label=colors)

    # Save/Open Map
    down_path = os.path.join(os.path.expanduser('~'), 'Downloads')
    map_file = os.path.join(down_path, 'map_example.html')
    m.save(map_file)
    webbrowser.open(map_file)



"""
ssss


"""

import concurrent.futures
import re
import unicodedata
import urllib.request

import pandas as pd
import requests
from bs4 import BeautifulSoup
from lxml import html
from more_itertools import one
from requests_ip_rotator import ApiGateway

import open_geodata as geo


def keep_numbers(my_string):
    """
    Mantem apenas números
    '987978098098098'
    https://stackoverflow.com/questions/1249388/removing-all-non-numeric-characters-from-string-in-python

    :return: _description_
    :rtype: _type_
    """
    return re.sub('[^0-9]', '', my_string)


def strip_accents(my_string: str) -> str:
    """
    Retira acentos das palavras
    https://stackoverflow.com/questions/517923/what-is-the-best-way-to-remove-accents-normalize-in-a-python-unicode-string

    :param my_string: Texto com acento
    :type my_string: string
    :return: Texto sem acento
    :rtype: my_string
    """
    return ''.join(
        c
        for c in unicodedata.normalize('NFD', str(my_string))
        if unicodedata.category(c) != 'Mn'
    )


def adjust_columns(df: pd.DataFrame, column_ajust: str) -> pd.DataFrame:
    """
    Adiciona coluna à tabela, com sufixo _temp,
    com correções diversas para fins de "fazer bater"

    :param df: tabela bruta
    :return: tabela com coluna a mais
    """
    #

    # Coluna para ajustar
    col_temp = f'{column_ajust}_temp'

    # Para Bater
    df[col_temp] = df[column_ajust]
    df[col_temp] = df[col_temp].str.strip()
    df[col_temp] = df[col_temp].str.lower()
    df[col_temp] = df.apply(
        lambda x: strip_accents(x[col_temp]), axis='columns'
    )
    df[col_temp] = df[col_temp].str.replace('’', '')
    df[col_temp] = df[col_temp].str.replace('´', '')
    df[col_temp] = df[col_temp].str.replace("'", '')

    #
    dd_fix = {
        # Errado / Certo
        'rio grande de serra': 'rio grande da serra',
        'santa rosa de viterbo': 'santa rosa do viterbo',
        'santana do parnaiba': 'santana de parnaiba',
        'sao luis do paraitinga': 'sao luiz do paraitinga',
    }

    df[col_temp] = df[col_temp].rename(dd_fix, axis='rows')
    df = df.replace({col_temp: dd_fix})

    # Results
    df.info()
    df.head()
    return df


class QuemSomos:
    def __init__(self) -> None:

        # Get Source Page
        url = 'https://www.tjsp.jus.br/QuemSomos/QuemSomos/RegioesAdministrativasJudiciarias'
        content = urllib.request.urlopen(url=url).read()
        tree = html.fromstring(content)

        # Get Box of Circunscrição
        list_divs = tree.xpath("//div[contains(@style, 'background')]")

        list_dfs_rajs = []
        list_dfs_cjs = []

        for div in list_divs:
            # A partir da div, pega os "p"
            list_p = div.xpath('.//p')
            raj = list_p[0].text_content().strip()
            raj_list = re.split(pattern=r'[-–]', string=raj, maxsplit=0)
            raj_num = raj_list[0].strip()
            raj_regiao = raj_list[1].strip()
            juiz = list_p[1].text_content().strip().split(':')[1].strip()
            email = re.sub('[()]', '', list_p[2].text_content().strip())

            # Para cada div, pega o primeiro ul que encontramos
            list_ul = div.xpath('./..//ul')[0]
            list_ul = div.getnext()
            list_li = list_ul.xpath('.//li')

            dict_raj = {
                'raj_nome': raj,
                'raj_sigla': raj_num,
                'raj_regiao': raj_regiao,
                'juiz_diretor_nome': juiz,
                'juiz_diretor_email': email,
            }
            list_dfs_rajs.append(dict_raj)

            # Circunscrição Judiciária
            list_cjs = [x.text_content() for x in list_li]
            df = pd.DataFrame(data=list_cjs, columns=['comarca_cirscunscricao'])
            df['raj_sigla'] = raj_num
            list_dfs_cjs.append(df)

        # ddd
        df = pd.concat(list_dfs_cjs, ignore_index=True)
        self.df = df
        self.list_dfs_rajs = list_dfs_rajs

    def get_raj(self) -> pd.DataFrame:
        df_raj = pd.DataFrame(self.list_dfs_rajs)
        df_raj['raj_nome'] = df_raj['raj_nome'].str.replace('–', '-')
        df_raj['id_raj'] = df_raj['raj_sigla'].apply(lambda x: keep_numbers(x))
        df_raj['id_raj'] = df_raj['id_raj'].astype(int)
        df_raj = df_raj.sort_values(by='id_raj', ascending=True)
        df_raj = df_raj.reset_index(drop=True)

        df_raj = df_raj[
            [
                # RAJ
                'id_raj',
                'raj_nome',
                'raj_sigla',
                'raj_regiao',
                'juiz_diretor_nome',
                'juiz_diretor_email',
            ]
        ]
        return df_raj

    def get_cj(self) -> pd.DataFrame:
        # ddd
        df_cj = self.df.copy()
        df_cj[['comarca', 'cj_sigla']] = df_cj[
            'comarca_cirscunscricao'
        ].str.rsplit('-', n=1, expand=True)

        df_cj['cj_sigla'] = df_cj['cj_sigla'].str.strip()

        # Id RAJ
        df_cj['id_raj'] = df_cj['raj_sigla'].apply(lambda x: keep_numbers(x))
        df_cj['id_raj'] = df_cj['id_raj'].astype(int)

        # Id CJ
        df_cj['id_cj'] = df_cj['cj_sigla'].apply(lambda x: keep_numbers(x))
        df_cj.loc[df_cj['id_cj'] == '', 'id_cj'] = '0'
        df_cj['id_cj'] = df_cj['id_cj'].astype(int)

        df_cj['cj_nome'] = df_cj['cj_sigla'].replace(
            'CJ', 'Circunscrição Judiciária', regex=True
        )
        df_cj['cj_nome'] = df_cj['cj_nome'].str.strip()

        df_cj = df_cj[
            [
                # CJ
                'id_cj',
                'cj_sigla',
                'cj_nome',
                #'comarca_cirscunscricao',
                # Comarca
                #'comarca',
                'id_raj',
            ]
        ]

        df_cj = df_cj.drop_duplicates()
        df_cj = df_cj.sort_values(by='id_cj')
        df_cj = df_cj.reset_index(drop=True)
        df_cj = df_cj.drop_duplicates()

        # Results
        # df_cj.to_clipboard(index=False)
        # df_cj.info()
        # df_cj.head(20)
        return df_cj

    def get_comarcas(self):
        # ddd
        df_comarca = self.df.copy()
        df_comarca[['comarca', 'cj_sigla']] = df_comarca[
            'comarca_cirscunscricao'
        ].str.rsplit('-', n=1, expand=True)

        df_comarca['cj_sigla'] = df_comarca['cj_sigla'].str.strip()

        # Id RAJ
        # df_comarca['id_raj'] = df_comarca['raj_sigla'].apply(lambda x: keep_numbers(x))
        # df_comarca['id_raj'] = df_comarca['id_raj'].astype(int)

        # Id CJ
        df_comarca['id_cj'] = df_comarca['cj_sigla'].apply(
            lambda x: keep_numbers(x)
        )
        df_comarca.loc[df_comarca['id_cj'] == '', 'id_cj'] = '0'
        df_comarca['id_cj'] = df_comarca['id_cj'].astype(int)

        df_comarca['cj_nome'] = df_comarca['cj_sigla'].replace(
            'CJ', 'Circunscrição Judiciária', regex=True
        )
        df_comarca['cj_nome'] = df_comarca['cj_nome'].str.strip()

        df_comarca = df_comarca[
            [
                # CJ
                'comarca',
                'id_cj',
                #'cj_sigla',
                #'cj_nome',
                #'comarca_cirscunscricao',
                # Comarca
                #'id_raj',
            ]
        ]

        df_comarca = df_comarca.drop_duplicates()

        # Renomear
        df_comarca = df_comarca.rename(
            {
                'comarca': 'comarca_tjsp',
            },
            axis='columns',
        )

        # Ordena
        df_comarca = df_comarca.iloc[
            df_comarca['comarca_tjsp'].str.normalize('NFKD').argsort()
        ]
        df_comarca = df_comarca.reset_index(drop=True)

        # Aplica strip em todo o dataframe
        df_comarca = df_comarca.map(
            lambda x: x.strip() if isinstance(x, str) else x
        )

        # Reordena as colunas
        df_comarca = df_comarca[['id_cj', 'comarca_tjsp']]

        # Results
        # df_comarca.to_clipboard(index=False)
        # df_comarca.info()
        # df_comarca.head(20)
        return df_comarca


class TJSP:
    def __init__(self) -> None:
        df_mun = geo.load_dataset(db='sp', name='tab.municipio_nome')
        self.lista_municipios = df_mun['municipio_nome']
        self.list_termos = self.list_terms_search()

    @property
    def get_max_caracteres_numero(self) -> int:
        return max([len(x) for x in self.lista_municipios])

    def get_lista_municipios_tjsp(self, contain):
        """
        Pesquisa de municípios a partir de alguns caracteres.
        A função sempre retorna 10 itens.
        A cada caractere, o número de registros afunila!

        Exemplo de uso:
        df = get_lista_municipios_tjsp('Santos')

        :param municipio: _description_
        :type municipio: _type_
        :raises Exception: _description_
        """
        if len(contain) < 3:
            raise Exception(
                'A pesquisa de município deve ter mais de 3 caracteres'
            )

        r = requests.post(
            url='https://www.tjsp.jus.br/AutoComplete/ListarMunicipios',
            json={'texto': contain},
        )
        if r.json() == 'listaVazia':
            pass

        else:
            df = pd.DataFrame(r.json())
            df = df.rename(
                mapper={
                    'Codigo': 'id_municipio_tjsp',
                    'Descricao': 'municipio_tjsp',
                },
                axis='columns',
            )
            return df

    def list_terms_search(self):
        """
        A partir do número de caracteres máximo dos 645 municícípios,
        é possível ver todas as combinações de 3 caracteres
        """
        # list_dfs = []
        list_termos = []
        for i in range(self.get_max_caracteres_numero)[3:]:
            lista_municipios_temp = list(
                set([mun[:i] for mun in self.lista_municipios if len(mun) >= i])
            )
            for search_text in lista_municipios_temp:
                list_termos.append(search_text)

        list_termos = list(set(list_termos))
        print(f'São {len(list_termos)} termos para pesquisa')
        return list_termos

    def search_terms(self, threads=4) -> pd.DataFrame:

        with concurrent.futures.ThreadPoolExecutor(
            max_workers=threads
        ) as executor:
            temp = executor.map(
                self.get_lista_municipios_tjsp, self.list_termos
            )
            df_tjsp = pd.concat(list(temp), ignore_index=True)

        return df_tjsp

    def search_terms_aws(
        self, aws_access_key_id, aws_secret_access_key
    ) -> pd.DataFrame:
        """
        sss
        """

        # Cria Gateway
        gateway = ApiGateway(
            site='https://www.tjsp.jus.br',
            access_key_id=aws_access_key_id,
            access_key_secret=aws_secret_access_key,
            regions=['sa-east-1'],
            verbose=True,
        )
        gateway.pool_connections = 5
        gateway.pool_maxsize = 5
        gateway.start()

        try:
            # Cria Session
            session = requests.Session()
            session.mount(prefix='https://www.tjsp.jus.br', adapter=gateway)

            # Parameters
            # MAX_THREADS = 4
            list_dfs = []
            # list_futures = []

            # Em 23.01.2025 tentei o uso do API
            for term in self.list_termos:
                # print(term)
                df = self.get_lista_municipios_tjsp(contain=term)
                list_dfs.append(df)

            # Crio a tabela
            df_tjsp = pd.concat(list_dfs, ignore_index=True)

            # Results
            return df_tjsp

        except Exception as e:
            raise Exception(e)

        finally:
            # Encerra o worker
            gateway.shutdown()

    def adjust_data(self, df):

        # Ajusta a tabela
        df_tjsp = df.drop_duplicates()
        df_tjsp = df_tjsp.sort_values(by='municipio_tjsp')
        df_tjsp = df_tjsp.iloc[
            df_tjsp['municipio_tjsp'].str.normalize('NFKD').argsort()
        ]
        df_tjsp = df_tjsp.reset_index(drop=True)

        num_municipios = len(df_tjsp)
        if num_municipios != 645:
            raise Exception(f'Falta Município! Temos {num_municipios}')

        # Resultados
        # df_tjsp.info()
        # df_tjsp.head()
        return df_tjsp


class ListaTelefonica:
    def __init__(self) -> None:
        pass

    def get_lista_unidades_tjsp(self, cod_municipio: int):
        """
        Pega a lista de unidades (Fóruns) de um determinado Município,
        a partir do Código do Município do TJSP
        :param cod_municipio: _description_
        :type cod_municipio: _type_
        :return: _description_
        """

        # Requests
        r: requests.Response = requests.post(
            'https://www.tjsp.jus.br/ListaTelefonica/RetornarResultadoBusca',
            json={'parmsEntrada': cod_municipio, 'codigoTipoBusca': 1},
            timeout=60,
        )

        # BS4
        soup = BeautifulSoup(r.text, 'html.parser')
        text_comarca = soup.find_all('h4')
        if text_comarca == []:
            raise Exception('Erro')

        else:
            text_comarca = one(text_comarca)
            text_comarca = text_comarca.text

            #
            comarca = text_comarca.split(' - ')[0]
            raj = text_comarca.strip().split(' - ')[-1]
            # print(raj)

            # comarca = comarca.split('está jurisdicionado à Comarca ')
            comarca = comarca.replace('Município ', '')
            comarca = comarca.replace('está jurisdicionado à Comarca', ' | ')
            comarca = comarca.replace('da Comarca', ' | ')
            # print(comarca)

            mun = comarca.strip().split(' | ')[0]
            com = comarca.strip().split(' | ')[-1]

            if mun.strip() == com.strip():
                comarca_sede = 1
            else:
                comarca_sede = 0

            # print(text_comarca)
            # print(text_comcarca.split('jurisdicionado à comarca '))

        lista_unidades = [x.text for x in soup.find_all('span')]

        return pd.DataFrame(
            {
                'id_municipio_tjsp': cod_municipio,
                'raj': raj.strip(),
                'municipio_tjsp': mun.strip(),
                'comarca_tjsp': com.strip(),
                'comarca_sede': comarca_sede,
                'unidades': lista_unidades,
            }
        )


if __name__ == '__main__':
    tjsp = QuemSomos()
    print(tjsp.df.head())

    df_raj = tjsp.get_raj()
    print(df_raj.head())

    df_cj = tjsp.get_cj()
    print(df_cj.head())

    df_comarca = tjsp.get_comarcas()
    print(df_comarca.head())
